@@ -23,9 +23,9 @@ summary_DD_TMB <- function(Assessment) {
                         stringsAsFactors = FALSE)
   rownames(derived) <- c("B0", "N0", "MSY", "UMSY", "BMSY")
 
-  if(conv) {
+  if(!is.character(SD)) {
     model_estimates <- summary(SD)
-    model_estimates <- model_estimates[model_estimates[, 2] > 0, ]
+    model_estimates <- model_estimates[is.na(model_estimates[, 2]) || model_estimates[, 2] > 0, ]
   } else {
     model_estimates <- SD
   }
