@@ -25,9 +25,10 @@ summary_SCA <- function(Assessment) {
   derived <- data.frame(Value = Value, Description = Description, stringsAsFactors = FALSE)
   rownames(derived) <- c("VB0", "SSB0", "MSY", "UMSY", "VBMSY", "SSBMSY")
 
-  if(conv) {
+  if(!is.character(SD)) {
     model_estimates <- summary(SD)[rownames(summary(SD)) != "log_rec_dev" & rownames(summary(SD)) != "log_early_rec_dev", ]
-    model_estimates <- model_estimates[model_estimates[, 2] > 0, ]
+    model_estimates <- model_estimates[is.na(model_estimates[, 2]) || model_estimates[, 2] > 0, ]
+    if(length(SE_Dev) == 0) SE_Dev <- rep(NA, length(Dev))
     dev_estimates <- cbind(Dev, SE_Dev)
     rownames(dev_estimates) <- paste0("log_rec_dev_", names(Dev))
     model_estimates <- rbind(model_estimates, dev_estimates)
@@ -249,7 +250,7 @@ generate_plots_SCA <- function(Assessment, save_figure = FALSE, save_dir = tempd
     plot_composition(Year2, Obs_CAA, Fit_CAA, plot_type = 'bubble_residuals', bubble_adj = 35)
     dev.off()
     assess.file.caption <- rbind(assess.file.caption,
-                               c("assess_age_comps_bubble_resids.png", "Age composition bubble plot of residuals (black are negative, white are positive)."))
+                               c("assess_age_comps_bubble_resids.png", "Bubble plot of Pearson residuals for age compositions (black are negative, white are positive)."))
   }
 
   plot_composition(Year2, Obs_CAA, Fit_CAA, N = info$data$CAA_n[ind_valid], plot_type = 'annual')
@@ -755,7 +756,7 @@ plot_yield_SCA <- function(data, report, umsy, msy, u.vector = seq(0, 1, 0.01),
   Brec <- report$Brec
 
   EPR <- Req <- NA
-  solveMSY <- function(logit_U, SR) {
+  solveMSY <- function(logit_U) {
     U <- ilogit(logit_U)
     surv <- exp(-M) * (1 - vul * U)
     NPR <- c(1, cumprod(surv[1:(maxage-1)]))
@@ -763,14 +764,14 @@ plot_yield_SCA <- function(data, report, umsy, msy, u.vector = seq(0, 1, 0.01),
     EPR <<- sum(NPR * mat * weight)
     if(SR == "BH") Req <<- (Arec * EPR - 1)/(Brec * EPR)
     if(SR == "Ricker") Req <<- log(Arec * EPR)/(Brec * EPR)
-    CPR <- vul * U * NPR
-    Yield <- Req * sum(CPR * weight)
+    YPR <- vul * U * NPR * weight
+    Yield <- Req * sum(YPR)
     return(-1 * Yield)
   }
 
   Biomass <- Yield <- R <- rep(NA, length(u.vector))
   for(i in 1:length(u.vector)) {
-    Yield[i] <- -1 * solveMSY(log(u.vector[i]/(1 - u.vector[i])), SR = SR)
+    Yield[i] <- -1 * solveMSY(logit(u.vector[i]))
     R[i] <- Req
     Biomass[i] <- EPR * Req
   }
